"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Server = void 0;
var fastify_1 = __importDefault(require("fastify"));
var socket_io_1 = require("socket.io");
var log_1 = require("./log");
var uuid_1 = require("uuid");
var redis_1 = require("redis");
var ts_nats_1 = require("ts-nats");
var Server = (function () {
    function Server(options) {
        this.options = options;
        this.sockets = {};
    }
    Server.prototype.init = function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.checkRegisteredApp().then(function () { return __awaiter(_this, void 0, void 0, function () {
                var timer, _a;
                var _b;
                return __generator(this, function (_c) {
                    switch (_c.label) {
                        case 0:
                            timer = log_1.Log.loading('Waiting started server');
                            this.app = fastify_1.default(this.options.server);
                            if (this.options.subscribers.redis)
                                this.redis = (_b = this.options.redis.client) !== null && _b !== void 0 ? _b : new redis_1.RedisClient(this.options.redis.options);
                            if (!this.options.subscribers.nats) return [3, 2];
                            _a = this;
                            return [4, ts_nats_1.connect(this.options.nats)];
                        case 1:
                            _a.nats = _c.sent();
                            _c.label = 2;
                        case 2:
                            this.io = new socket_io_1.Server(this.app.server, this.options.socketio);
                            return [4, this.registerPlugins()];
                        case 3:
                            _c.sent();
                            this.middlewares();
                            resolve(timer);
                            return [2];
                    }
                });
            }); }).catch(reject);
        });
    };
    Server.prototype.authorizeRequests = function (req, next) {
        var appId = req.headers['x-app-id'];
        var appSecret = req.headers['x-app-secret'];
        if (this.options.app.id === appId &&
            this.options.app.secret === appSecret &&
            uuid_1.validate(appId) && uuid_1.version(appId) === 4) {
            return next();
        }
        return next(new Error('Application invalid.'));
    };
    Server.prototype.checkRegisteredApp = function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var valid = true;
            if (!_this.options.app.id || !_this.options.app.secret) {
                valid = false;
                reject('Application not registered!');
            }
            var id = _this.options.app.id;
            var secret = _this.options.app.secret;
            if (!uuid_1.validate(id) && uuid_1.version(id) === 4) {
                valid = false;
                reject('The app id is not correct.');
            }
            if (secret.length <= 0) {
                valid = false;
                reject('Please set app secret!');
            }
            if (valid) {
                resolve(valid);
            }
        });
    };
    Server.prototype.registerPlugins = function () {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4, this.app.register(require('middie'))];
                    case 1:
                        _a.sent();
                        return [4, this.app.register(require('fastify-formbody'))];
                    case 2:
                        _a.sent();
                        return [4, this.app.register(require('fastify-multipart'), {
                                attachFieldsToBody: true
                            })];
                    case 3:
                        _a.sent();
                        return [2];
                }
            });
        });
    };
    Server.prototype.middlewares = function () {
        var _this = this;
        var _a;
        this.app.use(require('cors')((_a = this.options.server.cors) !== null && _a !== void 0 ? _a : {}));
        this.app.use(function (req, res, next) { return _this.authorizeRequests(req, next); });
        this.io.use(function (req, next) { return _this.authorizeRequests(req.handshake, next); });
    };
    return Server;
}());
exports.Server = Server;
//# sourceMappingURL=server.js.map