"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RedisSubscriber = void 0;
var log_1 = require("../log");
var RedisSubscriber = (function () {
    function RedisSubscriber(server, options) {
        this.server = server;
        this.options = options;
    }
    RedisSubscriber.prototype.subscribe = function (callback) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var _a;
            var prefix = (_a = _this.options.redis.prefix) !== null && _a !== void 0 ? _a : '';
            _this.server.on('pmessage', function (pattern, channel, message) {
                try {
                    message = JSON.parse(message);
                    if (message.event && message.data) {
                        if (_this.options.dev) {
                            log_1.Log.default("Channel: " + channel);
                            log_1.Log.default("Event: " + message.event);
                        }
                        callback(channel.substring(prefix.length), message);
                    }
                }
                catch (e) {
                    if (_this.options.dev) {
                        log_1.Log.error('No JSON message');
                    }
                    reject(e);
                }
            });
            _this.server.psubscribe(prefix + "*", function (err, count) {
                if (err) {
                    if (_this.options.dev) {
                        log_1.Log.error('Redis could not subscribe.');
                    }
                    reject();
                }
                resolve(true);
            });
        });
    };
    RedisSubscriber.prototype.unsubscribe = function () {
        return new Promise(function (resolve, reject) {
            resolve(true);
        });
    };
    return RedisSubscriber;
}());
exports.RedisSubscriber = RedisSubscriber;
//# sourceMappingURL=redis-subscriber.js.map